//---------------------------------------------------------------------------

#include <vcl.h>

#pragma hdrstop

#include "ProgrammableButton.h"
#include "ButtonConf.h"
#pragma package(smart_init)

namespace
{
	AnsiString GetFullImgName(AnsiString name)
	{
		AnsiString fname;
		fname.sprintf("%s\\img\\%s", ExtractFileDir(Application->ExeName).c_str(), name.c_str());
		return fname;
	}
	void LoadBitmap(Graphics::TBitmap *bmp, AnsiString name)
	{
		try
		{
			AnsiString fname = GetFullImgName(name);
			if (FileExists(fname))
			{
				bmp->LoadFromFile(fname);
			}
		} catch(...) {}
	}
}

//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
//
static inline void ValidCtrCheck(TProgrammableButton *)
{
	new TProgrammableButton(NULL, new TImageList(NULL));
}

__fastcall TProgrammableButton::TProgrammableButton(TComponent* Owner, TImageList* imgList)
	: TPanel(Owner), imgList(imgList), state(DIALOG_INFO_UNKNOWN), raised(true),
	bmpIdle(NULL), bmpTerminated(NULL), bmpEarly(NULL), bmpConfirmed(NULL)
{
	assert(imgList);

    //this->Color = clYellow;

    this->DoubleBuffered = true;

	this->Height = 32;
	this->Margins->Left = 0;
	this->Margins->Right = 0;
	this->Margins->Top = 0;
	this->Margins->Bottom = 0;
	this->AlignWithMargins = true;

	OnMouseEnter = MouseEnter;
	OnMouseLeave = MouseLeave;

	label = new TLabel(this);
	label->Parent = this;
	label->Left = 20;
	label->Top = 8;
	//label->AutoSize = false;
	//label->Width = this->Width - label->Left - 5;
	label->OnMouseEnter = MouseEnter;
	label->OnMouseLeave = MouseLeave;

	label2 = new TLabel(this);
	label2->Parent = this;
	label2->Left = 20;
	label2->Top = 26;
	//label2->AutoSize = false;
	//label2->Width = this->Width - label->Left - 5;
	label2->OnMouseEnter = MouseEnter;
	label2->OnMouseLeave = MouseLeave;
	label2->Visible = false;

	image = new TImage(this);
	image->Parent = this;
	image->Left = 2;
	image->Top = 8;
	image->Width = 16;
	image->Height = 16;
	image->Picture->Bitmap->PixelFormat = pf24bit;
	image->Transparent = true;
	image->OnMouseEnter = MouseEnter;
	image->OnMouseLeave = MouseLeave;

	bmpIdle = new Graphics::TBitmap();
	bmpTerminated = new Graphics::TBitmap();
	bmpEarly = new Graphics::TBitmap();
	bmpConfirmed = new Graphics::TBitmap();
}

__fastcall TProgrammableButton::~TProgrammableButton()
{
	delete bmpIdle;
	delete bmpTerminated;
	delete bmpEarly;
	delete bmpConfirmed;
}

//---------------------------------------------------------------------------
namespace ProgrammableButton
{
	void __fastcall PACKAGE Register()
	{
		TComponentClass classes[1] = {__classid(TProgrammableButton)};
		RegisterComponents("Samples", classes, 0);
	}
}
//---------------------------------------------------------------------------

void __fastcall TProgrammableButton::MouseEnter(TObject *Sender)
{
	Lower();
}

void __fastcall TProgrammableButton::MouseLeave(TObject *Sender)
{
	Raise();
}

void TProgrammableButton::SetConfig(const ButtonConf &cfg)
{
	label->Caption = cfg.caption.c_str();
	user = cfg.number.c_str();
	if (cfg.noIcon)
	{
		image->Width = 0;
		label->Left = 2;		
	}
	else
	{
		image->Width = 16;
		label->Left = 20;
	}
	Height = cfg.height;
	label->Top = (Height - label->Height)/2;
	image->Top = label->Top;

	this->Margins->Top = cfg.marginTop;
	this->Margins->Bottom = cfg.marginBottom;

	this->Color = static_cast<TColor>(cfg.backgroundColor);

	LoadBitmap(bmpIdle, cfg.imgIdle.c_str());
	LoadBitmap(bmpTerminated, cfg.imgTerminated.c_str());
	LoadBitmap(bmpEarly, cfg.imgEarly.c_str());
	LoadBitmap(bmpConfirmed, cfg.imgConfirmed.c_str());

	SetState(DIALOG_INFO_UNKNOWN);
	SetPresenceState(PRESENCE_UNKNOWN, "");
}

void TProgrammableButton::UpdateCallbacks(void)
{
	image->OnClick = OnClick;
	label->OnClick = OnClick;

	image->OnDblClick = OnDblClick;
	label->OnDblClick = OnDblClick;
}

void TProgrammableButton::SetState(enum dialog_info_status state)
{
	this->state = state;
	image->Picture = NULL;	// clear picture
	image->Picture->Bitmap->PixelFormat = pf24bit;
	switch (state)
	{
	case DIALOG_INFO_TERMINATED:
		//imgList->GetBitmap(1, image->Picture->Bitmap);
		image->Picture->Bitmap = bmpTerminated;
		break;
	case DIALOG_INFO_EARLY:
		//imgList->GetBitmap(2, image->Picture->Bitmap);
		image->Picture->Bitmap = bmpEarly;
		break;
	case DIALOG_INFO_CONFIRMED:
		//imgList->GetBitmap(3, image->Picture->Bitmap);
		image->Picture->Bitmap = bmpConfirmed;
		break;
	default: // DIALOG_INFO_UNKNOWN included
		//imgList->GetBitmap(0, image->Picture->Bitmap);
		image->Picture->Bitmap = bmpIdle;
	}
}

void TProgrammableButton::SetPresenceState(enum presence_status state, AnsiString note)
{
	this->presence_state = state;
	image->Picture = NULL;	// clear picture
	image->Picture->Bitmap->PixelFormat = pf24bit;
	switch (state)
	{
	case PRESENCE_OPEN:	// "Online"
		//imgList->GetBitmap(1, image->Picture->Bitmap);
		image->Picture->Bitmap = bmpTerminated;
		break;
	case PRESENCE_CLOSED:	// "Offline"
		//imgList->GetBitmap(2, image->Picture->Bitmap);
		image->Picture->Bitmap = bmpEarly;
		break;
	case PRESENCE_BUSY:
		//imgList->GetBitmap(3, image->Picture->Bitmap);
		image->Picture->Bitmap = bmpConfirmed;
		break;
	default: // PRESENCE_UNKNOWN included
		//imgList->GetBitmap(0, image->Picture->Bitmap);
		image->Picture->Bitmap = bmpIdle;
	}

	label2->Caption = note;
	if (note != "")
	{
		SetLines(2);
	}
	else
	{
		SetLines(1);
	}
}

void TProgrammableButton::SetMwiState(int newMsg, int oldMsg)
{
	AnsiString caption;
	caption.sprintf("%d / %d", newMsg, oldMsg);
	label->Caption = caption;
	if (newMsg)
		image->Picture->Bitmap = bmpEarly;
	else
		image->Picture->Bitmap = bmpIdle;
}

void TProgrammableButton::SetDown(bool state)
{
	down = state;
	if (down)
	{
		Lower();
	}
	else
	{
		Raise();
	}
}

void TProgrammableButton::Lower(void)
{
	if (raised == false)
		return;
	raised = false;			
	BevelOuter = bvLowered;
}

void TProgrammableButton::Raise(void)
{
	if (raised)
		return;
	if (down == false)
	{
        raised = true;
		BevelOuter = bvRaised;
	}		
}

void TProgrammableButton::SetLines(int cnt)
{
	if (cnt == 2)
	{
		image->Top = (Height - image->Height)/2;

		//label->Top = (Height - label->Height)/3;
		//label2->Top = (Height - label->Height)*2/3;

		label->Top = (Height - label->Height - label2->Height)/3;
		label2->Top = label->Top + label->Height + label->Top;

		label2->Visible = true;
	}
	else
	{
		label->Top = (Height - label->Height)/2;
		image->Top = (Height - image->Height)/2;
		label2->Visible = false;
	}
}



